"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const jsx_runtime_1 = require("react/jsx-runtime");
const React = (0, tslib_1.__importStar)(require("react"));
const react_dom_1 = (0, tslib_1.__importDefault)(require("react-dom"));
const prop_types_1 = (0, tslib_1.__importDefault)(require("prop-types"));
const react_popper_1 = require("react-popper");
const classnames_1 = (0, tslib_1.__importDefault)(require("classnames"));
const utils_1 = require("../utils");
const lodash_1 = require("lodash");
const DropdownToggle_1 = (0, tslib_1.__importDefault)(require("./DropdownToggle"));
const DropdownMenu_1 = (0, tslib_1.__importDefault)(require("./DropdownMenu"));
const DropdownItem_1 = (0, tslib_1.__importDefault)(require("./DropdownItem"));
const Dropdown_styles_1 = require("./Dropdown.styles");
class Dropdown extends React.Component {
    constructor() {
        super(...arguments);
        this.getContainer = () => {
            return react_dom_1.default.findDOMNode(this);
        };
        this.addEvents = () => {
            ["click", "touchstart", "keyup"].forEach((event) => document.addEventListener(event, this.handleDocumentClick, true));
        };
        this.removeEvents = () => {
            ["click", "touchstart", "keyup"].forEach((event) => document.removeEventListener(event, this.handleDocumentClick, true));
        };
        this.handleDocumentClick = (e) => {
            if (e && (e.which === 3 || (e.type === "keyup" && e.which !== utils_1.keyCodes.tab)))
                return;
            const container = this.getContainer();
            if (container &&
                container.contains(e.target) &&
                container !== e.target &&
                (e.type !== "keyup" || e.which === utils_1.keyCodes.tab)) {
                return;
            }
            this.toggle(e);
        };
        this.handleKeyDown = (e) => {
            if (utils_1.keyCodes.tab === e.which ||
                (/button/i.test(e.target.tagName) && [utils_1.keyCodes.space, utils_1.keyCodes.enter].includes(e.which)) ||
                /input|textarea|select/i.test(e.target.tagName)) {
                return;
            }
            e.preventDefault();
            if (this.props.disabled)
                return;
            const container = this.getContainer();
            if ((e.which === utils_1.keyCodes.space || e.which === utils_1.keyCodes.enter) &&
                this.props.isOpen &&
                container !== e.target) {
                e.target.click();
            }
            if (e.which === utils_1.keyCodes.esc ||
                (!this.props.isOpen && ![utils_1.keyCodes.down, utils_1.keyCodes.up].includes(e.which))) {
                this.toggle(e);
                if (container) {
                    const expanded = container.querySelector("[aria-expanded]");
                    expanded && expanded.focus();
                }
                return;
            }
            const menuClass = "dropdown-menu";
            const itemClass = "dropdown-item";
            const disabledClass = "disabled";
            const items = Array.from(container.querySelectorAll(`.${menuClass} .${itemClass}:not(.${disabledClass})`)).filter((v) => v && v.offsetParent);
            if (!items.length)
                return;
            let index = -1;
            const charPressed = String.fromCharCode(e.which).toLowerCase();
            for (let i = 0; i < items.length; i += 1) {
                const firstLetter = items[i].textContent && items[i].textContent[0].toLowerCase();
                if (firstLetter === charPressed || items[i] === e.target) {
                    index = i;
                    break;
                }
            }
            if (e.which === utils_1.keyCodes.up && index > 0) {
                index -= 1;
            }
            if (e.which === utils_1.keyCodes.down && index < items.length - 1) {
                index += 1;
            }
            if (index < 0) {
                index = 0;
            }
            items[index].focus();
        };
        this.handleProps = () => {
            if (this.props.isOpen) {
                this.addEvents();
            }
            else {
                this.removeEvents();
            }
        };
        this.toggle = (e) => {
            if (this.props.disabled) {
                return e && e.preventDefault();
            }
            return this.props.toggle(e);
        };
    }
    getChildContext() {
        const toggle = (e) => {
            const isDropdownToggle = e.target.classList.contains("dropdown-toggle");
            if (this.props.parentToggle && this.props.isOpen && !isDropdownToggle) {
                this.props.parentToggle(e);
            }
            this.props.toggle(e);
        };
        return {
            toggle: toggle,
            isOpen: this.props.isOpen,
            direction: this.props.direction,
            inNavbar: this.props.inNavbar,
        };
    }
    componentDidMount() {
        this.handleProps();
    }
    componentDidUpdate(prevProps) {
        if (this.props.isOpen !== prevProps.isOpen) {
            this.handleProps();
        }
    }
    componentWillUnmount() {
        this.removeEvents();
    }
    render() {
        const _a = (0, lodash_1.omit)(this.props, ["toggle", "parentToggle", "disabled", "inNavbar"]), { className, isOpen, group, size, nav, setActiveFromChild, active, addonType, direction } = _a, attrs = (0, tslib_1.__rest)(_a, ["className", "isOpen", "group", "size", "nav", "setActiveFromChild", "active", "addonType", "direction"]);
        attrs.tag = attrs.tag || (nav ? "li" : "div");
        let subItemIsActive = false;
        if (setActiveFromChild) {
            React.Children.map(this.props.children[1].props.children, (dropdownItem) => {
                if (dropdownItem && dropdownItem.props.active)
                    subItemIsActive = true;
            });
        }
        const classes = (0, classnames_1.default)(className, "react-dropdown", direction !== "down" && `drop${direction}`, nav && active ? "active" : false, setActiveFromChild && subItemIsActive ? "active" : false, {
            [`input-group-${addonType}`]: addonType,
            "btn-group": group,
            [`btn-group-${size}`]: !!size,
            dropdown: !group && !addonType,
            show: isOpen,
            "nav-item": nav,
        });
        return ((0, jsx_runtime_1.jsx)(react_popper_1.Manager, { children: (0, jsx_runtime_1.jsx)(Dropdown_styles_1.DropdownContainer, Object.assign({}, attrs, { className: classes, onKeyDown: this.handleKeyDown, role: this.props.inNavbar ? "presentation" : "menu" }), void 0) }, void 0));
    }
}
Dropdown.Toggle = DropdownToggle_1.default;
Dropdown.Menu = DropdownMenu_1.default;
Dropdown.Item = DropdownItem_1.default;
Dropdown.defaultProps = {
    isOpen: false,
    direction: "down",
    nav: false,
    active: false,
    addonType: false,
    inNavbar: false,
    setActiveFromChild: false,
};
Dropdown.childContextTypes = {
    toggle: prop_types_1.default.func.isRequired,
    isOpen: prop_types_1.default.bool.isRequired,
    direction: prop_types_1.default.oneOf(["up", "down", "left", "right"]).isRequired,
    inNavbar: prop_types_1.default.bool.isRequired,
};
exports.default = Dropdown;
