"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const jsx_runtime_1 = require("react/jsx-runtime");
const React = (0, tslib_1.__importStar)(require("react"));
const lodash_1 = require("lodash");
const BarLoader_1 = (0, tslib_1.__importDefault)(require("react-spinners/BarLoader"));
const styled_components_1 = (0, tslib_1.__importDefault)(require("styled-components"));
const InfiniteListItem_1 = (0, tslib_1.__importDefault)(require("./InfiniteListItem"));
const react_hotkeys_1 = require("react-hotkeys");
const keyboardNavigation_1 = require("./keyboardNavigation");
const Root = styled_components_1.default.div `
	overflow: hidden;
`;
const List = styled_components_1.default.ul `
	box-sizing: border-box;
	list-style: none;
	margin: 0;
	padding: 0;
`;
const InfiniteListContainer = styled_components_1.default.div `
	box-sizing: border-box;
	overflow: auto;
	height: calc(100% - 4px);
	-webkit-overflow-scrolling: touch;
`;
const LoaderContainer = styled_components_1.default.div `
	box-sizing: border-box;
	overflow: hidden;
	height: 4px;
	display: flex;
`;
const LoadButton = styled_components_1.default.button `
	box-sizing: border-box;
	border: 0;
	padding: 0;
	width: 100%;
	background: #2180c0;
	> .caret {
		color: #fff;
	}
`;
class InfiniteList extends React.Component {
    constructor(props) {
        super(props);
        this.listRef = React.createRef();
        this.getScrollableTarget = () => {
            if (this.props.scrollableTarget instanceof HTMLElement)
                return this.props.scrollableTarget;
            if (typeof this.props.scrollableTarget === "string") {
                return document.getElementById(this.props.scrollableTarget);
            }
            if (this.props.scrollableTarget === null) {
                console.warn(`You are trying to pass scrollableTarget but it is null. This might happen because the element may not have been added to DOM yet.`);
            }
            return null;
        };
        this.updateCanScroll = () => {
            const canScroll = !(this.el && this.listRef.current) || this.el.offsetHeight < this.listRef.current.offsetHeight;
            if (this.state.canScroll !== canScroll) {
                this.setState({ canScroll });
            }
            (0, lodash_1.delay)(this.updateCanScroll, 50);
        };
        this.next = () => {
            var _a, _b;
            this.setState((ps) => (Object.assign(Object.assign({}, ps), { actionTriggered: true, showLoader: true })));
            (_b = (_a = this.props).next) === null || _b === void 0 ? void 0 : _b.call(_a);
        };
        this.onScrollListener = (event) => {
            setTimeout(() => this.props.onScroll && this.props.onScroll(event), 0);
            const target = this.props.height || this._scrollableNode
                ? event.target
                : document.documentElement.scrollTop
                    ? document.documentElement
                    : document.body;
            if (this.state.actionTriggered)
                return;
            const atBottom = this.isElementAtBottom(target);
            if (atBottom && this.props.hasMore && this.props.dataLength) {
                this.next();
            }
        };
        this.throttledOnScrollListener = (0, lodash_1.throttle)(this.onScrollListener, 150);
        this.state = {
            dataLength: props.dataLength,
            showLoader: false,
            actionTriggered: false,
            canScroll: false,
        };
        this.keyboardHandlers = props.keyboardNavigationEnabled
            ? (0, keyboardNavigation_1.handlers)(this.listRef, { pageSize: this.props.keyboardNavigationPageSize })
            : null;
    }
    isElementAtBottom(target) {
        const clientHeight = target === document.body || target === document.documentElement
            ? window.screen.availHeight
            : target.clientHeight;
        return target.scrollHeight - Math.ceil(target.scrollTop) <= clientHeight;
    }
    static getDerivedStateFromProps(props, state) {
        if (props.dataLength === state.dataLength)
            return null;
        return {
            dataLength: props.dataLength,
            showLoader: false,
            actionTriggered: false,
        };
    }
    componentDidMount() {
        this._scrollableNode = this.getScrollableTarget();
        this.el = this.props.height ? this._infScroll : this._scrollableNode || window;
        this.el.addEventListener("scroll", this.throttledOnScrollListener);
        this.updateCanScroll();
    }
    componentWillUnmount() {
        this.el.removeEventListener("scroll", this.throttledOnScrollListener);
    }
    render() {
        const style = {
            height: this.props.height || "auto",
        };
        const hasChildren = this.props.hasChildren ||
            !!(this.props.children && this.props.children.length);
        const loading = (!this.state.showLoader && !hasChildren && this.props.hasMore) ||
            (this.state.showLoader && this.props.hasMore);
        return ((0, jsx_runtime_1.jsxs)(Root, Object.assign({ style: style }, { children: [(0, jsx_runtime_1.jsx)(LoaderContainer, { children: (0, jsx_runtime_1.jsx)(BarLoader_1.default, { width: "100%", height: 4, color: this.props.spinnerColor, loading: loading }, void 0) }, void 0), (0, jsx_runtime_1.jsxs)(InfiniteListContainer, Object.assign({ className: this.props.className, ref: (infScroll) => (this._infScroll = infScroll) }, { children: [(0, jsx_runtime_1.jsx)(react_hotkeys_1.HotKeys, Object.assign({ keyMap: keyboardNavigation_1.keyMap, handlers: this.keyboardHandlers }, { children: (0, jsx_runtime_1.jsx)(List, Object.assign({ ref: this.listRef, as: this.props.tag, className: this.props.listClassName, role: "list" }, { children: this.props.children }), void 0) }), void 0), !loading && this.props.hasMore && !this.state.canScroll && ((0, jsx_runtime_1.jsx)(LoadButton, Object.assign({ onClick: this.next, className: this.props.loadButtonClassName }, { children: (0, jsx_runtime_1.jsx)("div", { className: "caret" }, void 0) }), void 0)), !this.props.hasMore && this.props.endMessage] }), void 0)] }), void 0));
    }
}
exports.default = InfiniteList;
InfiniteList.Item = InfiniteListItem_1.default;
InfiniteList.defaultProps = {
    spinnerColor: "#2180c0",
    keyboardNavigationEnabled: false,
    keyboardNavigationPageSize: 3,
};
