"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handlers = exports.keyMap = void 0;
const lodash_1 = require("lodash");
exports.keyMap = {
    down: "down",
    up: "up",
    left: "left",
    right: "right",
    pageDown: "pagedown",
    pageUp: "pageup",
    top: "home",
    bottom: "end",
};
const currentItem = (list) => (0, lodash_1.find)(list === null || list === void 0 ? void 0 : list.children, (x) => x === document.activeElement || x.contains(document.activeElement));
const itemsPerRow = (list) => {
    var _a, _b;
    if ((0, lodash_1.isEmpty)(list === null || list === void 0 ? void 0 : list.children)) {
        return 0;
    }
    else {
        const top = (_b = (_a = list === null || list === void 0 ? void 0 : list.children) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.getBoundingClientRect().top;
        return (0, lodash_1.takeWhile)(list === null || list === void 0 ? void 0 : list.children, (child) => child.getBoundingClientRect().top === top).length;
    }
};
const next = (step, stepCount, start) => {
    let remained = stepCount;
    let cursor = start;
    while (remained-- > 0) {
        const next = step(cursor);
        if ((0, lodash_1.isNil)(next)) {
            return cursor;
        }
        else if (remained === 0) {
            return next;
        }
        else {
            cursor = next;
        }
    }
};
const stepForward = (x) => x === null || x === void 0 ? void 0 : x.nextElementSibling;
const stepBackward = (x) => x === null || x === void 0 ? void 0 : x.previousElementSibling;
const handlers = (listRef, { pageSize = 3 }) => {
    const navigate = (nextItem) => (e) => {
        const item = nextItem(currentItem(listRef.current), itemsPerRow(listRef.current));
        if (item) {
            item.focus();
            e === null || e === void 0 ? void 0 : e.stopPropagation();
            e === null || e === void 0 ? void 0 : e.preventDefault();
        }
    };
    const left = navigate((start, itemsPerRow) => (itemsPerRow > 1 ? stepBackward(start) : undefined));
    const right = navigate((start, itemsPerRow) => (itemsPerRow > 1 ? stepForward(start) : undefined));
    const up = navigate((start, itemsPerRow) => next(stepBackward, itemsPerRow, start));
    const down = navigate((start, itemsPerRow) => next(stepForward, itemsPerRow, start));
    const pageDown = navigate((0, lodash_1.partial)(next, stepForward, pageSize));
    const pageUp = navigate((0, lodash_1.partial)(next, stepBackward, pageSize));
    const top = navigate(() => { var _a; return (0, lodash_1.first)((_a = listRef.current) === null || _a === void 0 ? void 0 : _a.children); });
    const bottom = navigate(() => { var _a; return (0, lodash_1.last)((_a = listRef.current) === null || _a === void 0 ? void 0 : _a.children); });
    return {
        left,
        right,
        up,
        down,
        pageDown,
        pageUp,
        top,
        bottom,
    };
};
exports.handlers = handlers;
