"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const jsx_runtime_1 = require("react/jsx-runtime");
const React = (0, tslib_1.__importStar)(require("react"));
const base_1 = (0, tslib_1.__importDefault)(require("react-select/base"));
const react_select_1 = require("react-select");
const classnames_1 = (0, tslib_1.__importDefault)(require("classnames"));
const isNil_1 = (0, tslib_1.__importDefault)(require("lodash/isNil"));
const isEmpty_1 = (0, tslib_1.__importDefault)(require("lodash/isEmpty"));
const map_1 = (0, tslib_1.__importDefault)(require("lodash/map"));
const find_1 = (0, tslib_1.__importDefault)(require("lodash/find"));
const some_1 = (0, tslib_1.__importDefault)(require("lodash/some"));
const findIndex_1 = (0, tslib_1.__importDefault)(require("lodash/findIndex"));
const debounce_1 = (0, tslib_1.__importDefault)(require("lodash/debounce"));
const flatten_1 = (0, tslib_1.__importDefault)(require("lodash/flatten"));
const differenceBy_1 = (0, tslib_1.__importDefault)(require("lodash/differenceBy"));
const size_1 = (0, tslib_1.__importDefault)(require("lodash/size"));
const isEqual_1 = (0, tslib_1.__importDefault)(require("lodash/isEqual"));
const isArray_1 = (0, tslib_1.__importDefault)(require("lodash/isArray"));
const lodash_1 = (0, tslib_1.__importDefault)(require("lodash"));
const initialCache = {
    options: [],
    hasMore: true,
    isLoading: false,
};
function memoizeLastSingleValueReturn(returnFunc, compareByFunc) {
    return function memoized(...args) {
        const val = returnFunc(...args);
        if ((0, isArray_1.default)(val)) {
            delete memoized.val;
            return val;
        }
        else if (compareByFunc(memoized.val) === compareByFunc(val)) {
            return memoized.val;
        }
        else {
            memoized.val = val;
            return val;
        }
    };
}
function adjustOptionsAndSelected(selectedOption, options) {
    if (selectedOption && !(0, isArray_1.default)(selectedOption)) {
        const valueIndex = (0, findIndex_1.default)(options, (o) => o.value === selectedOption.value);
        if (valueIndex > -1) {
            options[valueIndex] = selectedOption;
        }
        else {
            options.unshift(selectedOption);
        }
    }
}
class Lookup extends React.PureComponent {
    constructor(props) {
        super(props);
        this.activeValue = null;
        this.selectRef = React.createRef();
        this.onMenuClose = () => {
            this.setState({
                search: "",
                menuIsOpen: false,
                customOptions: [],
            });
        };
        this.onMenuOpen = () => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (this.state.menuIsOpen) {
                return;
            }
            const reloadOptions = !this.props.cacheInitialOptions || !(0, isEqual_1.default)(this.currentQueryParams, this.state.queryParams);
            yield this.setState({
                menuIsOpen: true,
                optionsCache: reloadOptions ? this.initialOptionsCache : this.state.optionsCache,
            });
            const { optionsCache } = this.state;
            if (this.props.minInputLength === 0 && !(0, some_1.default)(optionsCache[""].options)) {
                yield this.loadOptions();
            }
        });
        this.onInputChange = (search) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.setState({
                search,
                customOptions: this.props.customOptions ? this.getCustomOptions(search) : [],
            });
            const { optionsCache } = this.state;
            const shouldLoadOptions = !optionsCache[search] && (0, size_1.default)(search) >= (this.props.minInputLength || 0);
            if (shouldLoadOptions) {
                yield this.loadOptions();
            }
        });
        this.onMenuScrollToBottom = () => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const { search, optionsCache } = this.state;
            const currentOptions = optionsCache[search];
            if (currentOptions) {
                yield this.loadOptions();
            }
        });
        this.onChange = (option, meta) => {
            const { fullObjectValue, onChange, idSelector, isMulti, value: lookupValues, multiAsString, delimiter, map: lookupMap, duplicatedValuesIdSetter, } = this.props;
            if (fullObjectValue) {
                if (!isMulti) {
                    meta.value =
                        option &&
                            (option.custom
                                ? option.fullObjectValue
                                : !(0, isNil_1.default)(option.value) &&
                                    (0, find_1.default)(this.currentOptions.values, (x) => idSelector(x) === option.value));
                }
                else {
                    const availableValues = this.currentOptions.values
                        ? this.currentOptions.values.concat(lookupValues)
                        : lookupValues;
                    if (!multiAsString) {
                        meta.value = (0, some_1.default)(option)
                            ? option.reduce((values, o) => {
                                if (o.custom) {
                                    return [...values, o.fullObjectValue];
                                }
                                const value = (0, find_1.default)(availableValues, (x) => idSelector(x) === o.value);
                                return value ? [...values, value] : values;
                            }, [])
                            : [];
                        lookupMap && (meta.value = (0, map_1.default)(meta.value, lookupMap));
                    }
                    else {
                        meta.value = (option || []).map((o) => o.value).join(delimiter);
                    }
                }
            }
            onChange && onChange(this.stripOptions(option), meta);
            if (duplicatedValuesIdSetter && fullObjectValue) {
                this.adjustOptionsForSameValuesSelection(meta);
            }
        };
        this.stripOptions = (option) => {
            if (!option) {
                return option;
            }
            else {
                return (0, isArray_1.default)(option)
                    ? option.map(({ label, value }) => ({ label, value }))
                    : { label: option.label, value: option.value };
            }
        };
        this.onMultiValueLabelClick = (labelProps) => (e) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (!this.props.onValueClick) {
                return;
            }
            e.persist();
            const availableValues = (0, lodash_1.default)(this.state.optionsCache)
                .values()
                .flatMap((x) => x.values)
                .uniqBy(this.props.idSelector)
                .value();
            if (!(0, some_1.default)(availableValues)) {
                yield this.loadOptions();
            }
            const { fullObjectValue, idSelector } = this.props;
            const option = labelProps ? labelProps.data : this.activeValue;
            if (!option) {
                return;
            }
            const value = fullObjectValue
                ? option.custom
                    ? option.fullObjectValue
                    : (0, find_1.default)(availableValues, (x) => idSelector(x) === option.value)
                : this.stripOptions(option);
            this.props.onValueClick(e.target, value);
        });
        this.mapValue = () => {
            const { isMulti, value: initialValue, renderOption, renderSelection, idSelector, multiAsString, delimiter, fullObjectValue, } = this.props;
            const renderFn = renderSelection || renderOption;
            if (initialValue && renderFn) {
                if (!isMulti) {
                    return {
                        label: initialValue.label || renderFn(initialValue) || idSelector(initialValue),
                        value: initialValue.value || idSelector(initialValue),
                    };
                }
                if (!multiAsString) {
                    return initialValue.map((x) => ({
                        label: x.label || renderFn(x) || idSelector(x),
                        value: x.value || idSelector(x),
                        fullObjectValue: fullObjectValue && x,
                    }));
                }
                return initialValue
                    ? initialValue.split(delimiter).map((o) => {
                        const current = (0, find_1.default)(this.allOptionValues, (v) => v && idSelector(v) && idSelector(v).toString() === o);
                        return {
                            label: current ? renderFn(current) : o,
                            value: o,
                        };
                    })
                    : null;
            }
            else {
                return initialValue;
            }
        };
        this.getCustomOptions = (search) => {
            if (!(search && search.trim())) {
                return [];
            }
            else {
                const { idSelector, renderOption, customOptions, fullObjectValue } = this.props;
                const customValues = customOptions(search.trim());
                return customValues
                    ? (0, flatten_1.default)([customValues]).map((x) => ({
                        value: idSelector(x),
                        label: renderOption(x) || idSelector(x),
                        custom: true,
                        fullObjectValue: fullObjectValue && x,
                    }))
                    : [];
            }
        };
        this.MultiValueContainer = (containerProps) => ((0, jsx_runtime_1.jsx)("div", Object.assign({ className: "multivalue-container", role: "region", "aria-labelledby": this.props.ariaLabelledBy }, { children: (0, jsx_runtime_1.jsx)(react_select_1.components.MultiValueContainer, Object.assign({}, containerProps), void 0) }), void 0));
        this.MultiValue = (multiValueProps) => {
            const { selectionBindings, fullObjectValue, idSelector } = this.props;
            let classes = "";
            if (multiValueProps.isFocused) {
                this.activeValue = multiValueProps.data;
            }
            if (selectionBindings) {
                const value = fullObjectValue
                    ? (0, find_1.default)(this.allOptionValues, (x) => x && idSelector(x) === multiValueProps.data.value)
                    : multiValueProps.data.value;
                classes = (0, classnames_1.default)(selectionBindings(value));
            }
            return ((0, jsx_runtime_1.jsx)(react_select_1.components.MultiValue, Object.assign({}, multiValueProps, { className: (0, classnames_1.default)("multi-value", classes, {
                    active: multiValueProps.isFocused && this.state.focused,
                }) }), void 0));
        };
        this.MultiValueLabel = (labelProps) => {
            const onClick = this.onMultiValueLabelClick(labelProps);
            const { optionBindings } = this.props;
            const classes = (optionBindings === null || optionBindings === void 0 ? void 0 : optionBindings(labelProps.data.value)) || [];
            return ((0, jsx_runtime_1.jsx)("button", Object.assign({ type: "button", className: (0, classnames_1.default)("multi-value-button", ...classes), onClick: onClick, style: { border: "none", background: "rgb(230, 230, 230)" }, "aria-haspopup": "dialog", tabIndex: -1 }, { children: labelProps.data.label }), void 0));
        };
        this.Option = (optionProps) => {
            const { optionBindings, fullObjectValue, idSelector } = this.props;
            const value = fullObjectValue
                ? (0, find_1.default)(this.allOptionValues, (x) => x && idSelector(x) === optionProps.data.value)
                : optionProps.data.value;
            const classes = (0, classnames_1.default)(optionBindings(value));
            return (0, jsx_runtime_1.jsx)(react_select_1.components.Option, Object.assign({}, optionProps, { className: classes }), void 0);
        };
        this.onKeyDown = (e) => {
            const eventKey = e.key;
            switch (eventKey) {
                case "Enter":
                    if (this.state.menuIsOpen) {
                        this.onMultiValueLabelClick()(e);
                    }
                    break;
                case "ArrowLeft":
                case "ArrowRight":
                    return this.setState({ menuIsOpen: false });
                case "Escape":
                    this.props.onEscape && this.props.onEscape();
                    break;
                default:
                    break;
            }
        };
        this.setFocus = () => this.setState((ps) => ({ focused: !ps.focused }));
        this.state = {
            search: "",
            optionsCache: this.initialOptionsCache,
            menuIsOpen: false,
            customOptions: [],
            queryParams: this.currentQueryParams,
        };
        this.onMenuOpen = (0, debounce_1.default)(this.onMenuOpen, 0);
        this.mapValue = memoizeLastSingleValueReturn(this.mapValue, (o) => o && o.value);
    }
    static filterOption() {
        return true;
    }
    get initialOptionsCache() {
        const { options, fullObjectValue, multiAsString, value } = this.props;
        return {
            "": {
                isLoading: false,
                options: options || [],
                values: fullObjectValue && !multiAsString && value ? value : [],
                hasMore: true,
            },
        };
    }
    get currentQueryParams() {
        var _a, _b, _c;
        const oDataQuery = (_b = (_a = this.props).queryProvider) === null || _b === void 0 ? void 0 : _b.call(_a, "");
        return ((_c = oDataQuery === null || oDataQuery === void 0 ? void 0 : oDataQuery.settings) === null || _c === void 0 ? void 0 : _c.params) || {};
    }
    get allOptionValues() {
        return (0, flatten_1.default)((0, map_1.default)(this.state.optionsCache, (search) => search.values))
            .concat(this.props.specialOptionValues)
            .filter((x) => x);
    }
    get currentOptions() {
        const { search, optionsCache } = this.state;
        return optionsCache[search] || initialCache;
    }
    componentDidMount() {
        if (this.props.initMultiString) {
            this.initMultiStringValues();
        }
    }
    componentDidUpdate() {
        const { isMulti, multiAsString, value, fullObjectValue, idSelector, map: lookupMap } = this.props;
        if (isMulti && fullObjectValue && !multiAsString) {
            let newValues = value && (0, differenceBy_1.default)(value, this.allOptionValues || [], idSelector);
            lookupMap && (newValues = (0, map_1.default)(newValues, lookupMap));
            (0, some_1.default)(newValues) &&
                this.setState((prevState) => ({
                    optionsCache: Object.assign(Object.assign({}, prevState.optionsCache), { [prevState.search]: Object.assign(Object.assign({}, prevState.optionsCache[prevState.search]), { values: [...newValues, ...this.currentOptions.values] }) }),
                }));
        }
    }
    initMultiStringValues() {
        const { queryProvider, multiStringIdFilterString, includeMetadata } = this.props;
        if (!(0, isEmpty_1.default)(multiStringIdFilterString)) {
            queryProvider === null || queryProvider === void 0 ? void 0 : queryProvider("").filter(multiStringIdFilterString).withQuery({ includeMetadata }).fetchAllPages().then((results) => {
                this.setState((prevState) => ({
                    optionsCache: {
                        "": Object.assign(Object.assign({}, prevState.optionsCache[""]), { values: results }),
                    },
                }));
            });
        }
    }
    loadOptions() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (!this.props.queryProvider) {
                return;
            }
            const { search } = this.state;
            if (this.currentOptions.isLoading || !this.currentOptions.hasMore) {
                return;
            }
            yield this.setState((prevState) => ({
                search,
                optionsCache: Object.assign(Object.assign({}, prevState.optionsCache), { [search]: Object.assign(Object.assign({}, this.currentOptions), { isLoading: true }) }),
            }));
            try {
                const results = yield this.load(search, this.currentOptions.options);
                if (!results.options) {
                    results.options = [];
                }
                const hasMore = results.options.length > 0 && results.options.length === this.props.pageSize;
                yield this.setState((prevState) => ({
                    optionsCache: Object.assign(Object.assign({}, prevState.optionsCache), { [search]: Object.assign(Object.assign({}, this.currentOptions), { options: this.currentOptions.options.concat(results.options), values: this.props.fullObjectValue &&
                                (this.currentOptions.values
                                    ? results.values.concat(this.currentOptions.values)
                                    : results.values), hasMore, isLoading: false }) }),
                }));
            }
            catch (e) {
                yield this.setState((prevState) => ({
                    optionsCache: Object.assign(Object.assign({}, prevState.optionsCache), { [search]: Object.assign(Object.assign({}, this.currentOptions), { isLoading: false }) }),
                }));
            }
        });
    }
    load(inputValue, previousOptions) {
        const { queryProvider, pageSize, resultsFilter, renderOption, idSelector, errorMessage, includeMetadata, fullObjectValue, } = this.props;
        const oDataQuery = queryProvider(inputValue);
        return new Promise((resolve, reject) => {
            oDataQuery
                .withQuery({ includeMetadata })
                .take(pageSize)
                .skip(previousOptions ? previousOptions.length : 0)
                .fetchCollection()
                .then((results) => {
                this.setState({ queryParams: oDataQuery.settings.params });
                return resultsFilter ? results.value.filter(resultsFilter) : results.value;
            })
                .then((results) => resolve({
                options: results.map((x) => ({
                    value: idSelector(x),
                    label: renderOption(x) || idSelector(x),
                    fullObjectValue: fullObjectValue && x,
                })),
                values: results,
            }))
                .catch(() => reject(errorMessage));
        });
    }
    adjustOptionsForSameValuesSelection(meta) {
        const { duplicatedValuesIdSetter, idSelector } = this.props;
        const search = this.state.search;
        if (meta.action === "select-option" && meta.option) {
            const duplicatedOption = lodash_1.default.cloneDeep(meta.option);
            duplicatedValuesIdSetter(duplicatedOption.fullObjectValue);
            duplicatedOption.value = idSelector(duplicatedOption.fullObjectValue);
            const optionIndex = lodash_1.default.findIndex(this.currentOptions.options, meta.option);
            const options = [...this.currentOptions.options];
            const values = [...this.currentOptions.values];
            options.splice(optionIndex, 0, duplicatedOption);
            values.splice(optionIndex, 0, duplicatedOption.fullObjectValue);
            this.setState((prevState) => (Object.assign(Object.assign({}, prevState), { optionsCache: Object.assign(Object.assign({}, prevState.optionsCache), { [search]: Object.assign(Object.assign({}, this.currentOptions), { options,
                        values }) }) })));
        }
        if (meta.action === "remove-value") {
            lodash_1.default.each(this.state.optionsCache, (cache, searchTerm) => {
                const redundantOption = lodash_1.default.find(cache.options, (option) => {
                    var _a;
                    return idSelector(option.fullObjectValue) ===
                        idSelector((_a = meta.removedValue) === null || _a === void 0 ? void 0 : _a.fullObjectValue);
                });
                if (redundantOption) {
                    this.setState((prevState) => (Object.assign(Object.assign({}, prevState), { optionsCache: Object.assign(Object.assign({}, prevState.optionsCache), { [searchTerm]: Object.assign(Object.assign({}, cache), { options: lodash_1.default.without(cache.options, redundantOption) }) }) })));
                }
            });
        }
    }
    get customComponentRenderers() {
        const { minInputLength, optionBindings, components } = this.props;
        const renderers = Object.assign({ MultiValueLabel: this.MultiValueLabel, MultiValueContainer: this.MultiValueContainer, MultiValue: this.MultiValue }, components);
        if (minInputLength > 0) {
            renderers.DropdownIndicator = null;
        }
        if (optionBindings) {
            renderers.Option = this.Option;
        }
        return renderers;
    }
    render() {
        const { placeholder, className, isMulti, styles, theme, isClearable, menuPlacement, noOptionsMessage, loadingMessage, id, disabled, ariaLabel, ariaLabelledBy, inputId, minInputLength, openMenuOnFocus, menuPortalTarget, displayCustomOptionsOnTop, } = this.props;
        const { search, menuIsOpen, customOptions } = this.state;
        const options = displayCustomOptionsOnTop
            ? customOptions.concat(this.currentOptions.options)
            : this.currentOptions.options.concat(customOptions);
        const valueOption = this.mapValue();
        adjustOptionsAndSelected(valueOption, options);
        return ((0, jsx_runtime_1.jsx)(base_1.default, { id: id, inputId: inputId, value: valueOption, inputValue: search, menuIsOpen: menuIsOpen, onFocus: this.setFocus, onBlur: this.setFocus, onMenuClose: this.onMenuClose, onMenuOpen: this.onMenuOpen, onKeyDown: this.onKeyDown, onInputChange: this.onInputChange, onMenuScrollToBottom: this.onMenuScrollToBottom, isLoading: this.currentOptions.isLoading, options: options, ref: this.selectRef, onChange: this.onChange, placeholder: placeholder, className: (0, classnames_1.default)("lookup", className), isMulti: isMulti, styles: styles, theme: theme, isClearable: isClearable, menuPlacement: menuPlacement, menuPortalTarget: menuPortalTarget, noOptionsMessage: noOptionsMessage, loadingMessage: loadingMessage, components: this.customComponentRenderers, openMenuOnClick: minInputLength === 0 && !isMulti, openMenuOnFocus: openMenuOnFocus, isDisabled: disabled, tabSelectsValue: false, "aria-label": ariaLabel, "aria-labelledby": ariaLabelledBy, filterOption: Lookup.filterOption }, void 0));
    }
}
exports.default = Lookup;
Lookup.defaultProps = {
    pageSize: 30,
    delay: 300,
    idSelector: (x) => x.Id,
    options: null,
    isMulti: false,
    isClearable: true,
    minInputLength: 0,
    openMenuOnFocus: false,
    menuPlacement: "bottom",
    placeholder: "",
    cacheInitialOptions: true,
    styles: {
        control: (base, state) => {
            let border, boxShadow;
            if (state.isFocused) {
                border = "1px solid #66afe9;";
                boxShadow = "inset 0 1px 1px rgba(0,0,0,.075), 0 0 8px rgba(102,175,233,.6);";
            }
            else {
                border = "1px solid #ccc;";
                boxShadow = "inset 0 1px 1px rgba(0,0,0,.075);";
            }
            return Object.assign(Object.assign({}, base), { minHeight: 34, height: "auto", border: border, borderRadius: 4, boxShadow: boxShadow });
        },
        menu: (base) => (Object.assign(Object.assign({}, base), { zIndex: 9999 })),
        menuPortal: (base) => (Object.assign(Object.assign({}, base), { zIndex: 9999 })),
        indicatorsContainer: (base, state) => (Object.assign(Object.assign({}, base), { ":before": state.isDisabled && {
                font: "normal normal normal 14px/1 FontAwesome",
                content: `"\\f023"`,
            } })),
    },
    delimiter: ";",
    includeMetadata: false,
};
