"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const jsx_runtime_1 = require("react/jsx-runtime");
const React = (0, tslib_1.__importStar)(require("react"));
const react_dom_1 = (0, tslib_1.__importDefault)(require("react-dom"));
const extend_1 = (0, tslib_1.__importDefault)(require("lodash/extend"));
const lodash_1 = require("lodash");
const load_script_1 = (0, tslib_1.__importDefault)(require("load-script"));
const defaultConfig = {
    toolbar: [
        "font",
        "fontSize",
        "bold",
        "italic",
        "underline",
        "-",
        "numberedList",
        "bulletedList",
        "-",
        "outdent",
        "indent",
        "-",
        "justifyLeft",
        "justifyCenter",
        "justifyRight",
        "justifyBlock",
        "-",
        "link",
        "easyImage",
        "maximize",
        "preview",
    ],
    fontSize_sizes: [
        "8",
        "9",
        "10",
        "11",
        "12",
        "14",
        "16",
        "18",
        "20",
        "22",
        "24",
        "26",
        "28",
        "36",
        "48",
        "72",
    ]
        .map((size) => size + "/" + size + "pt")
        .join(";"),
    font_names: [
        "Arial/Arial, Helvetica, sans-serif",
        "Calibri/Calibri, Tahoma, sans-serif",
        "Courier New/Courier New, Courier, monospace",
        "Georgia/Georgia, serif;Lucida Sans Unicode/Lucida Sans Unicode, Lucida Grande, sans-serif",
        "Tahoma/Tahoma, Geneva, sans-serif",
        "Times New Roman/Times New Roman, Times, serif",
        "Verdana/Verdana, Geneva, sans-serif",
    ].join(";"),
    allowedContent: true,
    resize_enabled: false,
    forcePasteAsPlainText: false,
    applyInitialStyle: true,
    bodyClass: "",
    startupFocus: false,
    disableNativeSpellChecker: false,
};
class RichText extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.mounted = false;
        this.onChange = () => {
            var _a;
            const data = RichText.editorInstances[this.props.id].getData().trim();
            if (data !== this.props.value) {
                (_a = this.props.events) === null || _a === void 0 ? void 0 : _a["change"](data);
            }
        };
        this.onLoad = () => {
            const { id, config, value, events } = this.props;
            if (!this.mounted)
                return;
            if (!window.CKEDITOR) {
                console.error("CKEditor not found");
                return;
            }
            RichText.editorInstances[id] = window.CKEDITOR.appendTo(react_dom_1.default.findDOMNode(this), (0, extend_1.default)(defaultConfig, config), value);
            (0, lodash_1.forOwn)(events, (eventHandler, event) => {
                if (event === "change") {
                    RichText.editorInstances[id].on(event, (0, lodash_1.throttle)(this.onChange, this.props.throttle));
                }
                else {
                    RichText.editorInstances[id].on(event, eventHandler);
                }
            });
            if (config.applyInitialStyle) {
                RichText.editorInstances[id].on("selectionChange", (event) => {
                    const initialStyle = new window.CKEDITOR.style({
                        element: "span",
                        styles: {
                            "font-family": "Calibri, Tahoma, sans-serif",
                            "font-size": "11pt",
                        },
                    });
                    if (!RichText.editorInstances[id].getData() &&
                        !initialStyle.checkActive(event.data.path)) {
                        RichText.editorInstances[id].applyStyle(initialStyle);
                    }
                }, null, null, 100);
            }
            RichText.editorInstances[id].on("toHtml", (event) => {
                const cleanHtml = event.data.dataValue
                    .replace(/<style[^>]*>\s*<!--/, "<style>")
                    .replace(/-->\s*<\/style>/, "</style>");
                event.data.dataValue = cleanHtml;
            }, null, null, 1);
            if (config.prependLines) {
                RichText.editorInstances[id].on("instanceReady", () => {
                    this.prependContentWithLines();
                });
            }
        };
    }
    componentDidUpdate(prevProps) {
        if (RichText.editorInstances[this.props.id] &&
            RichText.editorInstances[this.props.id].getData().trim() !== this.props.value &&
            prevProps.value !== this.props.value) {
            RichText.editorInstances[this.props.id].setData(this.props.value);
        }
    }
    componentDidMount() {
        this.mounted = true;
        if (!window.CKEDITOR) {
            (0, load_script_1.default)(this.props.scriptUrl, this.onLoad);
        }
        else {
            this.onLoad();
        }
    }
    componentWillUnmount() {
        this.mounted = false;
        if (RichText.editorInstances[this.props.id]) {
            RichText.editorInstances[this.props.id].removeAllListeners();
            RichText.editorInstances[this.props.id].destroy();
        }
    }
    prependContentWithLines() {
        const prependText = new Array(this.props.config.prependLines + 1).join("\n");
        RichText.editorInstances[this.props.id].focus();
        const range = RichText.editorInstances[this.props.id].createRange();
        range.moveToElementEditablePosition(range.root, false);
        RichText.editorInstances[this.props.id].insertText(prependText);
        RichText.editorInstances[this.props.id].getSelection().selectRanges([range]);
    }
    render() {
        const { id, className } = this.props;
        return (0, jsx_runtime_1.jsx)("div", { id: id, className: className }, void 0);
    }
}
exports.default = RichText;
RichText.defaultProps = {
    id: "richText",
    config: {},
    events: {},
    scriptUrl: "https://cdn.ckeditor.com/4.18.0/standard/ckeditor.js",
    throttle: 500,
};
RichText.editorInstances = {};
