import * as React from "react";
import Select from "react-select/base";
import { StylesConfig } from "react-select/src/styles";
import { ThemeConfig } from "react-select/src/theme";
import { SelectComponentsConfig, SelectComponents } from "react-select/src/components";
import { MultiValueGenericProps } from "react-select/src/components/MultiValue";
import { ActionMeta, OptionTypeBase } from "react-select/src/types";
export declare type LookupOption = {
    value: any;
    label: string;
};
export declare type AllowSameOptionSettings = {
    uniqueIdFieldName: any;
    originalIdSelector: (value: any) => any;
};
export interface LookupProps {
    id?: string;
    inputId?: string;
    value?: any;
    disabled?: boolean;
    onChange?: (...args: any[]) => any;
    renderOption?: (value: any) => string;
    renderSelection?: (value: any) => string;
    idSelector?: (value: any) => any;
    queryProvider?: (filter: string) => any;
    resultsFilter?: (value: any) => boolean;
    pageSize?: number;
    delay?: number;
    errorMessage?: string;
    options?: LookupOption[];
    placeholder?: string;
    className?: string;
    isMulti?: boolean;
    styles?: StylesConfig<OptionTypeBase, boolean>;
    theme?: ThemeConfig;
    isClearable?: boolean;
    menuPlacement?: "bottom" | "top" | "auto";
    menuPortalTarget?: HTMLElement;
    noOptionsMessage?: (value?: any) => string | null;
    loadingMessage?: (value?: any) => string | null;
    fullObjectValue?: boolean;
    onValueClick?: (target: HTMLElement, value: any) => any;
    onEscape?: () => any;
    customOptions?: (...args: any[]) => any;
    ariaLabel?: string;
    ariaLabelledBy?: string;
    labels?: {
        delete?: string;
    };
    minInputLength?: number;
    multiAsString?: boolean;
    delimiter?: string;
    optionBindings?: (value: any) => string[];
    selectionBindings?: (value: any) => string[];
    initMultiString?: boolean;
    multiStringIdFilterString?: string;
    specialOptionValues?: any[];
    openMenuOnFocus?: boolean;
    components?: SelectComponentsConfig<OptionTypeBase, boolean>;
    includeMetadata?: boolean;
    map?: (value: any) => any;
    duplicatedValuesIdSetter?: (value: any) => any;
    displayCustomOptionsOnTop?: boolean;
    cacheInitialOptions?: boolean;
}
declare type State = {
    search: string;
    optionsCache: {
        [key: string]: {
            isLoading: boolean;
            options: any[];
            values: any;
            hasMore?: boolean;
        };
    };
    menuIsOpen: boolean;
    customOptions: any[];
    queryParams: any;
    focused?: boolean;
};
export default class Lookup extends React.PureComponent<LookupProps, State> {
    static defaultProps: {
        pageSize: number;
        delay: number;
        idSelector: (x: any) => any;
        options: null;
        isMulti: boolean;
        isClearable: boolean;
        minInputLength: number;
        openMenuOnFocus: boolean;
        menuPlacement: string;
        placeholder: string;
        cacheInitialOptions: boolean;
        styles: {
            control: (base: any, state: any) => any;
            menu: (base: any) => any;
            menuPortal: (base: any) => any;
            indicatorsContainer: (base: any, state: any) => any;
        };
        delimiter: string;
        includeMetadata: boolean;
    };
    activeValue: null;
    selectRef: React.RefObject<Select<any, false>>;
    static filterOption(): boolean;
    get initialOptionsCache(): {
        "": {
            isLoading: boolean;
            options: LookupOption[];
            values: any;
            hasMore: boolean;
        };
    };
    get currentQueryParams(): any;
    get allOptionValues(): any[];
    get currentOptions(): {
        isLoading: boolean;
        options: any[];
        values: any;
        hasMore?: boolean | undefined;
    };
    constructor(props: any);
    componentDidMount(): void;
    componentDidUpdate(): void;
    initMultiStringValues(): void;
    onMenuClose: () => void;
    onMenuOpen: () => Promise<void>;
    onInputChange: (search: any) => Promise<void>;
    onMenuScrollToBottom: () => Promise<void>;
    loadOptions(): Promise<void>;
    load(inputValue: any, previousOptions: any): Promise<{
        options: LookupOption[];
        values: any[];
    }>;
    onChange: (option: any, meta: ActionMeta<OptionTypeBase> & {
        value?: any;
    }) => void;
    adjustOptionsForSameValuesSelection(meta: ActionMeta<OptionTypeBase> & {
        value?: any;
    }): void;
    stripOptions: (option: LookupOption | LookupOption[]) => {
        label: string;
        value: any;
    }[] | {
        label: string;
        value: any;
    };
    onMultiValueLabelClick: (labelProps?: MultiValueGenericProps<any> | undefined) => (e: any) => Promise<void>;
    mapValue: () => any;
    getCustomOptions: (search: any) => {
        value: any;
        label: any;
        custom: boolean;
        fullObjectValue: any;
    }[];
    MultiValueContainer: (containerProps: any) => JSX.Element;
    MultiValue: (multiValueProps: any) => JSX.Element;
    MultiValueLabel: (labelProps: MultiValueGenericProps<any>) => JSX.Element;
    Option: (optionProps: any) => JSX.Element;
    onKeyDown: (e: any) => void;
    setFocus: () => void;
    get customComponentRenderers(): Partial<SelectComponents<OptionTypeBase, boolean>>;
    render(): JSX.Element;
}
export {};
